package org.example.xacml.client;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.net.URL;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Service;
import javax.xml.ws.WebServiceException;

import org.example.xacml.svc.SecureEndpoint;
import org.junit.Test;

public class TestClient {
	public final String TARGET_ENDPOINT_ADDRESS = "http://localhost:8080/sampleDomain";

	private SecureEndpoint getPort() throws Exception {
		URL wsdlURL = new URL(TARGET_ENDPOINT_ADDRESS + "?wsdl");
		QName serviceName = new QName("http://org.example.xacml.svc/sampleDomain", "SecureEndpointService");
		SecureEndpoint port = Service.create(wsdlURL, serviceName).getPort(SecureEndpoint.class);
		return port;
	}

	@Test
	public void testNegativeA() throws Exception {
		SecureEndpoint port = getPort();
		try {
			port.echo("Hello");
			fail("Expected: Invalid HTTP server response [401] - Unauthorized");
		} catch (WebServiceException ex) {
			// all good
		}
	}

	@Test
	public void testNegativeB() throws Exception {
		SecureEndpoint port = getPort();

		Map<String, Object> reqContext = ((BindingProvider) port).getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "alice");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "alice");
		
		try {
			port.echo("Hello");
			fail("Expected: Invalid HTTP server response [401] - Unauthorized");
		} catch (WebServiceException ex) {
			// all good
		}
	}

	@Test
	public void testPositiveEcho() throws Exception {
		SecureEndpoint port = getPort();

		Map<String, Object> reqContext = ((BindingProvider) port).getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "bob");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "bob");

		String retObj = port.echo("Hello");
		assertEquals("Hello", retObj);
	}
}
